'use strict';

const config = require('config');
const fs = require('fs-extra');
const path = require('path');

function limitPathElement(element, limit) {
    return element.split('/').map((component) => {
        if (config.library.truncate && component.length > limit) {
            return component.slice(0, limit - config.library.truncate.truncator.length) + config.library.truncate.truncator;
        }

        return component;
    }).join(path.sep);
}

function getPathElements(requestedFilepath) {
    const originalPathElements = path.parse(requestedFilepath);

    return {
        root: originalPathElements.root,
        dir: limitPathElement(originalPathElements.dir, config.library.truncate.limit),
        name: limitPathElement(originalPathElements.name, config.library.truncate.limit - originalPathElements.ext.length),
        ext: originalPathElements.ext,
    };
}

function pipeStreamToFile(target, stream, item) {
    const file = fs.createWriteStream(target);

    return new Promise((resolve, reject) => {
        stream.pipe(file);

        stream.on('error', reject);
        stream.on('end', () => {
            if (item && item.mux) {
                console.log(`Temporarily saved '${target}', queued for muxing`);
            } else {
                console.log('\x1b[32m%s\x1b[0m', `Saved '${target}'`);
            }

            resolve(target);
        });
    });
}

async function save(requestedFilepath, streamOrStreams, item) {
    const pathElements = getPathElements(requestedFilepath);
    const streams = [].concat(streamOrStreams); // allow for single stream argument

    await fs.ensureDir(pathElements.dir);

    return Promise.all(streams.map((stream, index) => {
        const target = path.join(pathElements.root, pathElements.dir, `${pathElements.name}${streams.length > 1 ? `-${index}` : ''}${pathElements.ext}`);

        return pipeStreamToFile(target, stream, item);
    }));
}

module.exports = save;
